﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace ForceDirectedWPF {
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window {

        Diagram mDiagram;
        Random mRandom;

        public MainWindow() {
            InitializeComponent();

            mDiagram = new Diagram();
            mRandom = new Random();
        }

        private void btnGenerate_Click(object sender, RoutedEventArgs e) {
            mDiagram.Clear();

            // create a basic, random diagram that is between 2 and 4 levels deep 
            // and has between 1 and 10 leaf nodes per branch
            Node node = new SpotNode(Colors.Black);
            mDiagram.AddNode(node);

            for (int i = 0; i < mRandom.Next(1, 10); i++) {
                Node child = new SpotNode(Colors.Navy);
                node.AddChild(child);

                for (int j = 0; j < mRandom.Next(0, 10); j++) {
                    Node grandchild = new SpotNode(Colors.Blue);
                    child.AddChild(grandchild);

                    for (int k = 0; k < mRandom.Next(0, 10); k++) {
                        Node descendant = new SpotNode(Colors.CornflowerBlue);
                        grandchild.AddChild(descendant);
                    }
                }
            }

            // run the force-directed algorithm (asynchronously)
            btnGenerate.IsEnabled = false;
            Mouse.OverrideCursor = Cursors.Wait;
            Action<int?> operation = new Action<int?>(mDiagram.Arrange);
            AsyncCallback callback = result => {
                operation.EndInvoke(result);
                Dispatcher.Invoke(new Action(AfterArrange));
            };
            operation.BeginInvoke(0, callback, null);
        }

        private void AfterArrange() {
            myDrawingImage.Drawing = mDiagram.MakeDrawing(new Rect(0, 0, 100, 100));
            btnGenerate.IsEnabled = true;
            Mouse.OverrideCursor = null;
        }
    }
}
